package com.ctg.ag.sdk.core.http;

package ..deTemp.1736055826119;

import com.ctg.ag.sdk.core.HttpClient;
import com.ctg.ag.sdk.core.constant.ParamPosition;
import com.ctg.ag.sdk.core.exception.SdkClientException;
import com.ctg.ag.sdk.core.http.ApacheIdleConnectionCleaner;
import com.ctg.ag.sdk.core.model.ApiCallBack;
import com.ctg.ag.sdk.core.model.BaseApiRequest;
import com.ctg.ag.sdk.core.model.BaseApiResponse;
import com.ctg.ag.sdk.core.model.BuilderParams;
import java.io.IOException;
import java.net.URISyntaxException;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.EntityBuilder;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.methods.RequestBuilder;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.config.ConnectionConfig;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.ConnectionKeepAliveStrategy;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import org.apache.http.util.EntityUtils;

public class ApacheHttpClient extends HttpClient {
  private static final String EXT_PARAM_KEY_BUILDER = "apache.httpclient.builder";
  
  private static final int DEFAULT_THREAD_KEEP_ALIVE_TIME = 60;
  
  private ExecutorService executorService;
  
  private CloseableHttpClient httpClient;
  
  private PoolingHttpClientConnectionManager connectionManager;
  
  private String host;
  
  private String scheme;
  
  public ApacheHttpClient(BuilderParams params) {
    super(params);
  }
  
  protected void init(BuilderParams params) {
    HttpClientBuilder builder;
    if (params.containsExtParam("apache.httpclient.builder")) {
      builder = (HttpClientBuilder)params.getExtParam("apache.httpclient.builder");
    } else {
      builder = HttpClientBuilder.create();
    } 
    RequestConfig defaultConfig = RequestConfig.custom().setConnectTimeout((int)params.getConnectionTimeoutMillis()).setCookieSpec("ignoreCookies").setSocketTimeout((int)params.getReadTimeoutMillis()).setConnectionRequestTimeout((int)params.getWriteTimeoutMillis()).build();
    builder.setDefaultRequestConfig(defaultConfig);
    SSLContext sslContext = null;
    try {
      sslContext = (new SSLContextBuilder()).loadTrustMaterial((KeyStore)null, (TrustStrategy)new Object(this)).build();
      builder.setSSLSocketFactory(new SSLConnectionSocketFactory(sslContext, (HostnameVerifier)new Object(this)));
    } catch (NoSuchAlgorithmException e) {
      e.printStackTrace();
    } catch (KeyManagementException e) {
      e.printStackTrace();
    } catch (KeyStoreException e) {
      e.printStackTrace();
    } 
    Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create().register("http", PlainConnectionSocketFactory.getSocketFactory()).register("https", new SSLConnectionSocketFactory(sslContext, (HostnameVerifier)new Object(this))).build();
    this.connectionManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
    this.connectionManager.setMaxTotal(params.getMaxRequests());
    this.connectionManager.setDefaultMaxPerRoute(params.getMaxRequestsPerHost());
    this.connectionManager.setDefaultConnectionConfig(ConnectionConfig.custom().setCharset(Charset.forName("utf-8")).build());
    builder.setConnectionManager(this.connectionManager);
    ApacheIdleConnectionCleaner.registerConnectionManager(this.connectionManager, Long.valueOf(params.getMaxIdleTimeMillis()));
    if (params.getExecutorService() == null) {
      this.executorService = new ThreadPoolExecutor(0, params.getMaxRequests(), 60L, TimeUnit.SECONDS, new SynchronousQueue<>(), (ThreadFactory)new DeafultAsyncThreadFactory(this, null));
    } else {
      this.executorService = params.getExecutorService();
    } 
    if (params.getKeepAliveDurationMillis() > 0L)
      builder.setKeepAliveStrategy((ConnectionKeepAliveStrategy)new Object(this, params)); 
    this.httpClient = builder.build();
    this.host = params.getHost();
    this.scheme = params.getScheme().getValue();
  }
  
  private HttpUriRequest parseToHttpRequest(BaseApiRequest apiReq) {
    RequestBuilder builder = RequestBuilder.create(apiReq.getMethod().getName());
    try {
      URIBuilder uriBuilder = new URIBuilder();
      uriBuilder.setScheme((apiReq.getScheme() != null) ? apiReq.getScheme() : this.scheme);
      String host = (apiReq.getHost() != null) ? apiReq.getHost() : this.host;
      String[] split = host.split("/");
      uriBuilder.setHost(split[0]);
      apiReq.getGlobalParam().put("Host", uriBuilder.getHost());
      String path = ((split.length > 1) ? split[1] : "") + apiReq.getPath();
      if (!path.startsWith("/"))
        path = "/" + path; 
      uriBuilder.setPath(path);
      Map<String, List<String>> query = apiReq.getParam(new ParamPosition[] { ParamPosition.QUERY });
      if (query != null)
        for (Map.Entry<String, List<String>> entry : query.entrySet()) {
          for (String s : entry.getValue())
            uriBuilder.addParameter(entry.getKey(), s); 
        }  
      builder.setUri(uriBuilder.build());
    } catch (URISyntaxException e) {
      throw new SdkClientException("build http request uri failed", e);
    } 
    EntityBuilder bodyBuilder = EntityBuilder.create();
    bodyBuilder.setContentType(apiReq.getMethod().getContentType());
    Map<String, List<String>> body = apiReq.getParam(new ParamPosition[] { ParamPosition.BODY });
    if (body != null && body.size() != 0) {
      List<NameValuePair> paramList = new ArrayList<>();
      for (Map.Entry<String, List<String>> entry : body.entrySet()) {
        for (String s : entry.getValue())
          paramList.add(new BasicNameValuePair(entry.getKey(), s)); 
      } 
      bodyBuilder.setParameters(paramList);
      builder.setEntity(bodyBuilder.build());
    } else if (apiReq.getBody() != null) {
      bodyBuilder.setBinary(apiReq.getBody());
      builder.setEntity(bodyBuilder.build());
    } 
    Map<String, List<String>> header = apiReq.getParam(new ParamPosition[] { ParamPosition.HEAD });
    for (Map.Entry<String, List<String>> entry : header.entrySet()) {
      for (String s : entry.getValue())
        builder.addHeader(entry.getKey(), s); 
    } 
    Map<String, String> global = apiReq.getGlobalParam();
    for (Map.Entry<String, String> entry : global.entrySet())
      builder.addHeader(entry.getKey(), entry.getValue()); 
    return builder.build();
  }
  
  private BaseApiResponse parseToApiResponse(HttpResponse httpResponse, BaseApiResponse result) throws IOException {
    result.setStatusCode(httpResponse.getStatusLine().getStatusCode());
    if (httpResponse.getEntity() != null) {
      Header contentType = httpResponse.getEntity().getContentType();
      if (contentType != null)
        result.setContentType(contentType.getValue()); 
      result.setBody(EntityUtils.toByteArray(httpResponse.getEntity()));
    } else {
      Header contentType = httpResponse.getFirstHeader("Content-Type");
      result.setContentType(contentType.getValue());
    } 
    result.setHeaders(new HashMap<>());
    for (Header header : httpResponse.getAllHeaders())
      result.getHeaders().put(header.getName(), header.getValue()); 
    Header message = httpResponse.getFirstHeader("x-ag-message");
    if (message != null) {
      result.setMessage(message.getValue());
    } else {
      result.setMessage(httpResponse.getStatusLine().getReasonPhrase());
    } 
    return result;
  }
  
  public final BaseApiResponse syncInvoke(BaseApiRequest apiRequest) {
    HttpUriRequest httpRequest = parseToHttpRequest(apiRequest);
    CloseableHttpResponse httpResponse = null;
    try {
      httpResponse = this.httpClient.execute(httpRequest);
      BaseApiResponse response = apiRequest.newResponse();
      return parseToApiResponse(httpResponse, response);
    } catch (IOException e) {
      throw new SdkClientException(e);
    } finally {
      try {
        if (httpResponse != null)
          httpResponse.close(); 
      } catch (Exception exception) {}
    } 
  }
  
  public <REQ extends BaseApiRequest, RESP extends BaseApiResponse> Future<RESP> asyncInvoke(REQ request, ApiCallBack<REQ, RESP> callback) {
    return this.executorService.submit((Callable<RESP>)new Object(this, (BaseApiRequest)request, callback));
  }
  
  public void shutdown() {
    this.executorService.shutdown();
    ApacheIdleConnectionCleaner.removeConnectionManager(this.connectionManager);
    this.connectionManager.shutdown();
    if (this.httpClient != null)
      try {
        this.httpClient.close();
      } catch (Exception exception) {} 
  }
}
